import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.HashSet;

import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;

public class SetDemo
{
    private Set<Person> people;
    
    public SetDemo() {
        people = new HashSet<>();
    }
    
    public Collection<Person> getPeople() {
        return Collections.unmodifiableCollection(people);
    }
    
    public void init() {
        
        people.add(new Person("Jan Novak", "2014/11/20"));
        people.add(new Person("Jan Novak", "2014/11/20"));
        people.add(new Person("Jan Novak", "1970/11/20"));
        people.add(new Person("Jan Novak", "2014/10/21") );
        
        //getPeople().add(new Person("Pavel Novotny", "2014/10/21"));
        
        System.out.println("Contains?: "
           +people.contains(new Person("Jan Novak", "2014/10/21")));
        }
    
    public static void main(String[] args)
    {
        SetDemo d = new SetDemo();
        d.init();
        
        sortedSetDemo();
        
        SortedMap<Person, Double> salaries = new TreeMap<>();
        salaries.put(new Person("Jan Novak", "2014/11/20"), 10000.0);
        salaries.put(new Person("Pavel Novotny", "1970/11/20"), 35000.0);
        salaries.put(new Person("Ales Vacha", "1960/11/20"), 35000.0);
        salaries.put(new Person("Jan Novak", "1970/11/20"), 50000.0);

        
        salaries.put(new Person("Jan Novak", "2014/11/20"), 20000.0);

        double salary = salaries.get(new Person("Jan Novak", "2014/11/20"));
        System.out.println("Jan Novak salary="+salary);
        
        for(Map.Entry<Person, Double> me: salaries.entrySet() ) {
            System.out.println("Entry "+me.getKey()+"=>"+me.getValue());
        }
        
     }
     public static void sortedSetDemo() {
        
        SortedSet<Person> persons = new TreeSet<>();
        persons.add(new Person("Jan Novak", "2014/11/20"));
        persons.add(new Person("Pavel Novotny", "1970/11/20"));
        persons.add(new Person("Ales Vacha", "1960/11/20"));
        persons.add(new Person("Jan Novak", "1970/11/20"));

        for(Person p: persons) {
            System.out.println("Person "+p);
        }
        

        }
     public static void sortedSetDemoComparator() {
        
        SortedSet<Person> persons = new TreeSet<>(new DateOfBirthComparator());
        persons.add(new Person("Jan Novak", "2014/11/20"));
        persons.add(new Person("Pavel Novotny", "1970/11/20"));
        persons.add(new Person("Ales Vacha", "1960/11/20"));
        persons.add(new Person("Jan Novak", "1970/11/20"));

        for(Person p: persons) {
            System.out.println("Person "+p);
        }
        

        }
}
class DateOfBirthComparator implements Comparator<Person> {
    public int compare(Person a, Person b) {
        int comp = a.getDate().compareTo(b.getDate());
        if(comp == 0) {
            return a.getName().compareTo(b.getName());
        } else return comp;
    }
}
class DateOfBirthComparatorDescending extends DateOfBirthComparator {
    public int compare(Person a, Person b) {
        return -super.compare(a, b);
    }
}

class Person implements Comparable<Person> {
    private String name;
    private String date;
    public Person(String name, String date) { this.name = name; this.date = date; }
    public String toString() { return name+" born "+date; }
    public int compareTo(Person p) {
        int comp = name.compareTo(p.name);
        if(comp == 0) {
            return date.compareTo(p.date);
        } else return comp;
    }
    public boolean equals(Object obj) {
        if(obj instanceof Person) {
            Person p = (Person)obj;
            return name.equals(p.name) && date.equals(p.date);
        } else return false;
    }
    public int hashCode() {
        return name.hashCode() ^ date.hashCode();
    }
    public String getDate() { return date; }
    public String getName() { return name; }
}